@@ -1,6 +1,6 @@
 /*
    CheMPS2: a spin-adapted implementation of DMRG for ab initio quantum chemistry
-   Copyright (C) 2013-2015 Sebastian Wouters
+   Copyright (C) 2013-2016 Sebastian Wouters
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -22,10 +22,12 @@
 #include <iostream>
 #include <sstream>
 #include <string>
+#include <climits>
 
 #include "FourIndex.h"
 #include "Lapack.h"
 #include "MyHDF5.h"
+#include "MPIchemps2.h"
 
 using namespace std;
 
@@ -289,6 +291,8 @@ long long CheMPS2::FourIndex::getPtrIrrepOrderOK(const int irrep_i, const int ir
    
 }
 
+int CheMPS2::FourIndex::get_irrep_size( const int irrep ) const{ return Isizes[ irrep ]; }
+
 long long CheMPS2::FourIndex::getPtrAllOK1(const int Icent, const int irrep_i, const int irrep_k, const int i, const int j, const int k, const int l) const{
 
    return storage[Icent][irrep_i][irrep_k][i + k*(k+1)/2][j-i] + l-k;
@@ -423,3 +427,15 @@ void CheMPS2::FourIndex::read(const std::string name){
 
 }
 
+#ifdef CHEMPS2_MPI_COMPILATION
+void CheMPS2::FourIndex::broadcast( const int ROOT ){
+
+   assert( arrayLength <= INT_MAX ); // To be able to broadcast
+   const int size = ( int )( arrayLength );
+   if ( size > 0 ){
+      MPIchemps2::broadcast_array_double( theElements, size, ROOT );
+   }
+
+}
+#endif
+
