@@ -1,6 +1,6 @@
 /*
    CheMPS2: a spin-adapted implementation of DMRG for ab initio quantum chemistry
-   Copyright (C) 2013-2015 Sebastian Wouters
+   Copyright (C) 2013-2016 Sebastian Wouters
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -21,8 +21,8 @@
 
 #include "Heff.h"
 #include "Lapack.h"
-#include "Gsl.h"
 #include "MPIchemps2.h"
+#include "Wigner.h"
 
 void CheMPS2::Heff::addDiagonal1A(const int ikappa, double * memHeffDiag, const Sobject * denS, TensorX * Xleft) const{
    int dimL = denBK->gCurrentDim(denS->gIndex(), denS->gNL(ikappa), denS->gTwoSL(ikappa), denS->gIL(ikappa));
@@ -121,7 +121,7 @@ void CheMPS2::Heff::addDiagonal2d3all(const int ikappa, double * memHeffDiag, co
    
 }
 
-void CheMPS2::Heff::addDiagonal2b3spin0(const int ikappa, double * memHeffDiag, const Sobject * denS, TensorC * Ctensor) const{
+void CheMPS2::Heff::addDiagonal2b3spin0(const int ikappa, double * memHeffDiag, const Sobject * denS, TensorOperator * Ctensor) const{
 
    int N1 = denS->gN1(ikappa);
    
@@ -150,7 +150,7 @@ void CheMPS2::Heff::addDiagonal2b3spin0(const int ikappa, double * memHeffDiag,
    
 }
 
-void CheMPS2::Heff::addDiagonal2c3spin0(const int ikappa, double * memHeffDiag, const Sobject * denS, TensorC * Ctensor) const{
+void CheMPS2::Heff::addDiagonal2c3spin0(const int ikappa, double * memHeffDiag, const Sobject * denS, TensorOperator * Ctensor) const{
 
    int N2 = denS->gN2(ikappa);
    
@@ -179,7 +179,7 @@ void CheMPS2::Heff::addDiagonal2c3spin0(const int ikappa, double * memHeffDiag,
    
 }
 
-void CheMPS2::Heff::addDiagonal2e3spin0(const int ikappa, double * memHeffDiag, const Sobject * denS, TensorC * Ctensor) const{
+void CheMPS2::Heff::addDiagonal2e3spin0(const int ikappa, double * memHeffDiag, const Sobject * denS, TensorOperator * Ctensor) const{
 
    int N1 = denS->gN1(ikappa);
    
@@ -208,7 +208,7 @@ void CheMPS2::Heff::addDiagonal2e3spin0(const int ikappa, double * memHeffDiag,
    
 }
 
-void CheMPS2::Heff::addDiagonal2f3spin0(const int ikappa, double * memHeffDiag, const Sobject * denS, TensorC * Ctensor) const{
+void CheMPS2::Heff::addDiagonal2f3spin0(const int ikappa, double * memHeffDiag, const Sobject * denS, TensorOperator * Ctensor) const{
 
    int N2 = denS->gN2(ikappa);
    
@@ -237,7 +237,7 @@ void CheMPS2::Heff::addDiagonal2f3spin0(const int ikappa, double * memHeffDiag,
    
 }
 
-void CheMPS2::Heff::addDiagonal2b3spin1(const int ikappa, double * memHeffDiag, const Sobject * denS, TensorD * Dtensor) const{
+void CheMPS2::Heff::addDiagonal2b3spin1(const int ikappa, double * memHeffDiag, const Sobject * denS, TensorOperator * Dtensor) const{
 
    int N1 = denS->gN1(ikappa);
    
@@ -259,7 +259,7 @@ void CheMPS2::Heff::addDiagonal2b3spin1(const int ikappa, double * memHeffDiag,
       int dimR     = denBK->gCurrentDim(theindex+2,NR,TwoSR,IR);
       
       int fase = phase(TwoSL + TwoSR + 2*TwoJ + ((N2==1)?1:0) - 1);
-      const double alpha = fase * (TwoJ+1) * sqrt(3.0*(TwoSL+1)) * gsl_sf_coupling_6j(TwoJ,TwoJ,2,1,1,((N2==1)?1:0)) * gsl_sf_coupling_6j(TwoJ,TwoJ,2,TwoSL,TwoSL,TwoSR);
+      const double alpha = fase * (TwoJ+1) * sqrt(3.0*(TwoSL+1)) * Wigner::wigner6j(TwoJ,TwoJ,2,1,1,((N2==1)?1:0)) * Wigner::wigner6j(TwoJ,TwoJ,2,TwoSL,TwoSL,TwoSR);
       
       double * Dblock = Dtensor->gStorage(NL,TwoSL,IL,NL,TwoSL,IL);
       for (int cntR=0; cntR<dimR; cntR++){
@@ -272,7 +272,7 @@ void CheMPS2::Heff::addDiagonal2b3spin1(const int ikappa, double * memHeffDiag,
    
 }
 
-void CheMPS2::Heff::addDiagonal2c3spin1(const int ikappa, double * memHeffDiag, const Sobject * denS, TensorD * Dtensor) const{
+void CheMPS2::Heff::addDiagonal2c3spin1(const int ikappa, double * memHeffDiag, const Sobject * denS, TensorOperator * Dtensor) const{
 
    int N2 = denS->gN2(ikappa);
    
@@ -294,7 +294,7 @@ void CheMPS2::Heff::addDiagonal2c3spin1(const int ikappa, double * memHeffDiag,
       int dimR     = denBK->gCurrentDim(theindex+2,NR,TwoSR,IR);
       
       int fase = phase(TwoSL + TwoSR + 2*TwoJ + ((N1==1)?1:0) - 1);
-      const double alpha = fase * (TwoJ+1) * sqrt(3.0*(TwoSL+1)) * gsl_sf_coupling_6j(TwoJ,TwoJ,2,1,1,((N1==1)?1:0)) * gsl_sf_coupling_6j(TwoJ,TwoJ,2,TwoSL,TwoSL,TwoSR);
+      const double alpha = fase * (TwoJ+1) * sqrt(3.0*(TwoSL+1)) * Wigner::wigner6j(TwoJ,TwoJ,2,1,1,((N1==1)?1:0)) * Wigner::wigner6j(TwoJ,TwoJ,2,TwoSL,TwoSL,TwoSR);
       
       double * Dblock = Dtensor->gStorage(NL,TwoSL,IL,NL,TwoSL,IL);
       for (int cntR=0; cntR<dimR; cntR++){
@@ -307,7 +307,7 @@ void CheMPS2::Heff::addDiagonal2c3spin1(const int ikappa, double * memHeffDiag,
    
 }
 
-void CheMPS2::Heff::addDiagonal2e3spin1(const int ikappa, double * memHeffDiag, const Sobject * denS, TensorD * Dtensor) const{
+void CheMPS2::Heff::addDiagonal2e3spin1(const int ikappa, double * memHeffDiag, const Sobject * denS, TensorOperator * Dtensor) const{
 
    int N1 = denS->gN1(ikappa);
    
@@ -329,7 +329,7 @@ void CheMPS2::Heff::addDiagonal2e3spin1(const int ikappa, double * memHeffDiag,
       int dimR     = denBK->gCurrentDim(theindex+2,NR,TwoSR,IR);
       
       int fase = phase(TwoSR + TwoSL + 2*TwoJ + ((N2==1)?1:0) + 1);
-      const double alpha = fase * (TwoJ+1) * sqrt(3.0*(TwoSR+1)) * gsl_sf_coupling_6j(TwoJ,TwoJ,2,1,1,((N2==1)?1:0)) * gsl_sf_coupling_6j(TwoJ,TwoJ,2,TwoSR,TwoSR,TwoSL);
+      const double alpha = fase * (TwoJ+1) * sqrt(3.0*(TwoSR+1)) * Wigner::wigner6j(TwoJ,TwoJ,2,1,1,((N2==1)?1:0)) * Wigner::wigner6j(TwoJ,TwoJ,2,TwoSR,TwoSR,TwoSL);
       
       double * Dblock = Dtensor->gStorage(NR,TwoSR,IR,NR,TwoSR,IR);
       for (int cntR=0; cntR<dimR; cntR++){
@@ -342,7 +342,7 @@ void CheMPS2::Heff::addDiagonal2e3spin1(const int ikappa, double * memHeffDiag,
    
 }
 
-void CheMPS2::Heff::addDiagonal2f3spin1(const int ikappa, double * memHeffDiag, const Sobject * denS, TensorD * Dtensor) const{
+void CheMPS2::Heff::addDiagonal2f3spin1(const int ikappa, double * memHeffDiag, const Sobject * denS, TensorOperator * Dtensor) const{
 
    int N2 = denS->gN2(ikappa);
    
@@ -364,7 +364,7 @@ void CheMPS2::Heff::addDiagonal2f3spin1(const int ikappa, double * memHeffDiag,
       int dimR     = denBK->gCurrentDim(theindex+2,NR,TwoSR,IR);
       
       int fase = phase(TwoSR + TwoSL + 2*TwoJ + ((N1==1)?1:0) + 1);
-      const double alpha = fase * (TwoJ+1) * sqrt(3.0*(TwoSR+1)) * gsl_sf_coupling_6j(TwoJ,TwoJ,2,1,1,((N1==1)?1:0)) * gsl_sf_coupling_6j(TwoJ,TwoJ,2,TwoSR,TwoSR,TwoSL);
+      const double alpha = fase * (TwoJ+1) * sqrt(3.0*(TwoSR+1)) * Wigner::wigner6j(TwoJ,TwoJ,2,1,1,((N1==1)?1:0)) * Wigner::wigner6j(TwoJ,TwoJ,2,TwoSR,TwoSR,TwoSL);
       
       double * Dblock = Dtensor->gStorage(NR,TwoSR,IR,NR,TwoSR,IR);
       for (int cntR=0; cntR<dimR; cntR++){
@@ -377,7 +377,7 @@ void CheMPS2::Heff::addDiagonal2f3spin1(const int ikappa, double * memHeffDiag,
    
 }
 
-void CheMPS2::Heff::addDiagonal2a3spin0(const int ikappa, double * memHeffDiag, const Sobject * denS, TensorC **** Ctensors, TensorF0 **** F0tensors) const{
+void CheMPS2::Heff::addDiagonal2a3spin0(const int ikappa, double * memHeffDiag, const Sobject * denS, TensorOperator **** Ctensors, TensorF0 **** F0tensors) const{
 
    #ifdef CHEMPS2_MPI_COMPILATION
    const int MPIRANK = MPIchemps2::mpi_rank();
@@ -495,7 +495,7 @@ void CheMPS2::Heff::addDiagonal2a3spin0(const int ikappa, double * memHeffDiag,
    
 }
 
-void CheMPS2::Heff::addDiagonal2a3spin1(const int ikappa, double * memHeffDiag, const Sobject * denS, TensorD **** Dtensors, TensorF1 **** F1tensors) const{
+void CheMPS2::Heff::addDiagonal2a3spin1(const int ikappa, double * memHeffDiag, const Sobject * denS, TensorOperator **** Dtensors, TensorF1 **** F1tensors) const{
 
    #ifdef CHEMPS2_MPI_COMPILATION
    const int MPIRANK = MPIchemps2::mpi_rank();
@@ -511,7 +511,7 @@ void CheMPS2::Heff::addDiagonal2a3spin1(const int ikappa, double * memHeffDiag,
    
    int TwoJ = denS->gTwoJ(ikappa);
    const int fase = phase(TwoSL+TwoSR+TwoJ+2);
-   const double alpha = fase * sqrt((TwoSR + 1)*(TwoSL + 1.0)) * gsl_sf_coupling_6j(TwoSL,TwoSR,TwoJ,TwoSR,TwoSL,2);
+   const double alpha = fase * sqrt((TwoSR + 1)*(TwoSL + 1.0)) * Wigner::wigner6j(TwoSL,TwoSR,TwoJ,TwoSR,TwoSL,2);
    
    int theindex = denS->gIndex();
    int ptr = denS->gKappa2index(ikappa);
