@@ -1,6 +1,6 @@
 /*
    CheMPS2: a spin-adapted implementation of DMRG for ab initio quantum chemistry
-   Copyright (C) 2013-2015 Sebastian Wouters
+   Copyright (C) 2013-2016 Sebastian Wouters
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -22,10 +22,10 @@
 
 #include "Heff.h"
 #include "Lapack.h"
-#include "Gsl.h"
 #include "MPIchemps2.h"
+#include "Wigner.h"
 
-void CheMPS2::Heff::addDiagram2a1spin0(const int ikappa, double * memS, double * memHeff, const Sobject * denS, TensorA **** Atensors, TensorS0 **** S0tensors, double * workspace) const{
+void CheMPS2::Heff::addDiagram2a1spin0(const int ikappa, double * memS, double * memHeff, const Sobject * denS, TensorOperator **** Atensors, TensorS0 **** S0tensors, double * workspace) const{
 
    #ifdef CHEMPS2_MPI_COMPILATION
    const int MPIRANK = MPIchemps2::mpi_rank();
@@ -58,8 +58,8 @@ void CheMPS2::Heff::addDiagram2a1spin0(const int ikappa, double * memS, double *
             if ( MPIchemps2::owner_absigma( l_alpha, l_beta ) == MPIRANK )
             #endif
             {
-               int ILdown = Irreps::directProd(IL,S0tensors[theindex-1][l_beta-l_alpha][theindex-1-l_beta]->gIdiff());
-               int IRdown = Irreps::directProd(IR,Atensors[theindex+1][l_beta-l_alpha][theindex+1-l_beta]->gIdiff());
+               int ILdown = Irreps::directProd(IL,S0tensors[theindex-1][l_beta-l_alpha][theindex-1-l_beta]->get_irrep());
+               int IRdown = Irreps::directProd(IR,Atensors[theindex+1][l_beta-l_alpha][theindex+1-l_beta]->get_irrep());
                int memSkappa = denS->gKappa(NL-2,TwoSL,ILdown,N1,N2,TwoJ,NR-2,TwoSR,IRdown);
                
                if (memSkappa!=-1){
@@ -95,8 +95,8 @@ void CheMPS2::Heff::addDiagram2a1spin0(const int ikappa, double * memS, double *
             if ( MPIchemps2::owner_absigma( l_gamma, l_delta ) == MPIRANK )
             #endif
             {
-               int ILdown = Irreps::directProd(IL,Atensors[theindex-1][l_delta-l_gamma][l_gamma-theindex]->gIdiff());
-               int IRdown = Irreps::directProd(IR,S0tensors[theindex+1][l_delta-l_gamma][l_gamma-theindex-2]->gIdiff());
+               int ILdown = Irreps::directProd(IL,Atensors[theindex-1][l_delta-l_gamma][l_gamma-theindex]->get_irrep());
+               int IRdown = Irreps::directProd(IR,S0tensors[theindex+1][l_delta-l_gamma][l_gamma-theindex-2]->get_irrep());
                int memSkappa = denS->gKappa(NL-2,TwoSL,ILdown,N1,N2,TwoJ,NR-2,TwoSR,IRdown);
                
                if (memSkappa!=-1){
@@ -126,7 +126,7 @@ void CheMPS2::Heff::addDiagram2a1spin0(const int ikappa, double * memS, double *
    
 }
 
-void CheMPS2::Heff::addDiagram2a2spin0(const int ikappa, double * memS, double * memHeff, const Sobject * denS, TensorA **** Atensors, TensorS0 **** S0tensors, double * workspace) const{
+void CheMPS2::Heff::addDiagram2a2spin0(const int ikappa, double * memS, double * memHeff, const Sobject * denS, TensorOperator **** Atensors, TensorS0 **** S0tensors, double * workspace) const{
 
    #ifdef CHEMPS2_MPI_COMPILATION
    const int MPIRANK = MPIchemps2::mpi_rank();
@@ -159,8 +159,8 @@ void CheMPS2::Heff::addDiagram2a2spin0(const int ikappa, double * memS, double *
             if ( MPIchemps2::owner_absigma( l_alpha, l_beta ) == MPIRANK )
             #endif
             {
-               int ILdown = Irreps::directProd(IL,S0tensors[theindex-1][l_beta-l_alpha][theindex-1-l_beta]->gIdiff());
-               int IRdown = Irreps::directProd(IR,Atensors[theindex+1][l_beta-l_alpha][theindex+1-l_beta]->gIdiff());
+               int ILdown = Irreps::directProd(IL,S0tensors[theindex-1][l_beta-l_alpha][theindex-1-l_beta]->get_irrep());
+               int IRdown = Irreps::directProd(IR,Atensors[theindex+1][l_beta-l_alpha][theindex+1-l_beta]->get_irrep());
                int memSkappa = denS->gKappa(NL+2,TwoSL,ILdown,N1,N2,TwoJ,NR+2,TwoSR,IRdown);
                
                if (memSkappa!=-1){
@@ -196,8 +196,8 @@ void CheMPS2::Heff::addDiagram2a2spin0(const int ikappa, double * memS, double *
             if ( MPIchemps2::owner_absigma( l_gamma, l_delta ) == MPIRANK )
             #endif
             {
-               int ILdown = Irreps::directProd(IL,Atensors[theindex-1][l_delta-l_gamma][l_gamma-theindex]->gIdiff());
-               int IRdown = Irreps::directProd(IR,S0tensors[theindex+1][l_delta-l_gamma][l_gamma-theindex-2]->gIdiff());
+               int ILdown = Irreps::directProd(IL,Atensors[theindex-1][l_delta-l_gamma][l_gamma-theindex]->get_irrep());
+               int IRdown = Irreps::directProd(IR,S0tensors[theindex+1][l_delta-l_gamma][l_gamma-theindex-2]->get_irrep());
                int memSkappa = denS->gKappa(NL+2,TwoSL,ILdown,N1,N2,TwoJ,NR+2,TwoSR,IRdown);
                
                if (memSkappa!=-1){
@@ -227,7 +227,7 @@ void CheMPS2::Heff::addDiagram2a2spin0(const int ikappa, double * memS, double *
    
 }
 
-void CheMPS2::Heff::addDiagram2a1spin1(const int ikappa, double * memS, double * memHeff, const Sobject * denS, TensorB **** Btensors, TensorS1 **** S1tensors, double * workspace) const{
+void CheMPS2::Heff::addDiagram2a1spin1(const int ikappa, double * memS, double * memHeff, const Sobject * denS, TensorOperator **** Btensors, TensorS1 **** S1tensors, double * workspace) const{
 
    #ifdef CHEMPS2_MPI_COMPILATION
    const int MPIRANK = MPIchemps2::mpi_rank();
@@ -259,7 +259,7 @@ void CheMPS2::Heff::addDiagram2a1spin1(const int ikappa, double * memS, double *
             if ((TwoSLdown>=0) && (TwoSRdown>=0) && (abs(TwoSLdown-TwoSRdown)<=TwoJ)){
             
                int fase = phase(TwoSRdown+TwoSL+TwoJ+2);
-               const double thefactor = fase * sqrt((TwoSR + 1)*(TwoSL + 1.0)) * gsl_sf_coupling_6j(TwoSLdown,TwoSRdown,TwoJ,TwoSR,TwoSL,2);
+               const double thefactor = fase * sqrt((TwoSR + 1)*(TwoSL + 1.0)) * Wigner::wigner6j( TwoSLdown, TwoSRdown, TwoJ, TwoSR, TwoSL, 2 );
       
                for (int l_alpha=0; l_alpha<theindex; l_alpha++){
                   for (int l_beta=l_alpha+1; l_beta<theindex; l_beta++){
@@ -268,8 +268,8 @@ void CheMPS2::Heff::addDiagram2a1spin1(const int ikappa, double * memS, double *
                      if ( MPIchemps2::owner_absigma( l_alpha, l_beta ) == MPIRANK )
                      #endif
                      {
-                        int ILdown = Irreps::directProd(IL,S1tensors[theindex-1][l_beta-l_alpha][theindex-1-l_beta]->gIdiff());
-                        int IRdown = Irreps::directProd(IR,Btensors[theindex+1][l_beta-l_alpha][theindex+1-l_beta]->gIdiff());
+                        int ILdown = Irreps::directProd(IL,S1tensors[theindex-1][l_beta-l_alpha][theindex-1-l_beta]->get_irrep());
+                        int IRdown = Irreps::directProd(IR,Btensors[theindex+1][l_beta-l_alpha][theindex+1-l_beta]->get_irrep());
                         int memSkappa = denS->gKappa(NL-2,TwoSLdown,ILdown,N1,N2,TwoJ,NR-2,TwoSRdown,IRdown);
                
                         if (memSkappa!=-1){
@@ -307,7 +307,7 @@ void CheMPS2::Heff::addDiagram2a1spin1(const int ikappa, double * memS, double *
             if ((TwoSLdown>=0) && (TwoSRdown>=0) && (abs(TwoSLdown-TwoSRdown)<=TwoJ)){
             
                int fase = phase(TwoSRdown+TwoSL+TwoJ+2);
-               const double thefactor = fase * sqrt((TwoSR + 1)*(TwoSL + 1.0)) * gsl_sf_coupling_6j(TwoSLdown,TwoSRdown,TwoJ,TwoSR,TwoSL,2);
+               const double thefactor = fase * sqrt((TwoSR + 1)*(TwoSL + 1.0)) * Wigner::wigner6j(TwoSLdown,TwoSRdown,TwoJ,TwoSR,TwoSL,2);
       
                for (int l_gamma=theindex+2; l_gamma<Prob->gL(); l_gamma++){
                   for (int l_delta=l_gamma+1; l_delta<Prob->gL(); l_delta++){
@@ -316,8 +316,8 @@ void CheMPS2::Heff::addDiagram2a1spin1(const int ikappa, double * memS, double *
                      if ( MPIchemps2::owner_absigma( l_gamma, l_delta ) == MPIRANK )
                      #endif
                      {
-                        int ILdown = Irreps::directProd(IL,Btensors[theindex-1][l_delta-l_gamma][l_gamma-theindex]->gIdiff());
-                        int IRdown = Irreps::directProd(IR,S1tensors[theindex+1][l_delta-l_gamma][l_gamma-theindex-2]->gIdiff());
+                        int ILdown = Irreps::directProd(IL,Btensors[theindex-1][l_delta-l_gamma][l_gamma-theindex]->get_irrep());
+                        int IRdown = Irreps::directProd(IR,S1tensors[theindex+1][l_delta-l_gamma][l_gamma-theindex-2]->get_irrep());
                         int memSkappa = denS->gKappa(NL-2,TwoSLdown,ILdown,N1,N2,TwoJ,NR-2,TwoSRdown,IRdown);
                
                         if (memSkappa!=-1){
@@ -350,7 +350,7 @@ void CheMPS2::Heff::addDiagram2a1spin1(const int ikappa, double * memS, double *
    
 }
 
-void CheMPS2::Heff::addDiagram2a2spin1(const int ikappa, double * memS, double * memHeff, const Sobject * denS, TensorB **** Btensors, TensorS1 **** S1tensors, double * workspace) const{
+void CheMPS2::Heff::addDiagram2a2spin1(const int ikappa, double * memS, double * memHeff, const Sobject * denS, TensorOperator **** Btensors, TensorS1 **** S1tensors, double * workspace) const{
 
    #ifdef CHEMPS2_MPI_COMPILATION
    const int MPIRANK = MPIchemps2::mpi_rank();
@@ -382,7 +382,7 @@ void CheMPS2::Heff::addDiagram2a2spin1(const int ikappa, double * memS, double *
             if ((TwoSLdown>=0) && (TwoSRdown>=0) && (abs(TwoSLdown-TwoSRdown)<=TwoJ)){
             
                int fase = phase(TwoSLdown+TwoSR+TwoJ+2);
-               const double thefactor = fase * sqrt((TwoSRdown + 1)*(TwoSLdown + 1.0)) * gsl_sf_coupling_6j(TwoSLdown,TwoSRdown,TwoJ,TwoSR,TwoSL,2);
+               const double thefactor = fase * sqrt((TwoSRdown + 1)*(TwoSLdown + 1.0)) * Wigner::wigner6j(TwoSLdown,TwoSRdown,TwoJ,TwoSR,TwoSL,2);
          
                for (int l_alpha=0; l_alpha<theindex; l_alpha++){
                   for (int l_beta=l_alpha+1; l_beta<theindex; l_beta++){
@@ -391,8 +391,8 @@ void CheMPS2::Heff::addDiagram2a2spin1(const int ikappa, double * memS, double *
                      if ( MPIchemps2::owner_absigma( l_alpha, l_beta ) == MPIRANK )
                      #endif
                      {
-                        int ILdown = Irreps::directProd(IL,S1tensors[theindex-1][l_beta-l_alpha][theindex-1-l_beta]->gIdiff());
-                        int IRdown = Irreps::directProd(IR,Btensors[theindex+1][l_beta-l_alpha][theindex+1-l_beta]->gIdiff());
+                        int ILdown = Irreps::directProd(IL,S1tensors[theindex-1][l_beta-l_alpha][theindex-1-l_beta]->get_irrep());
+                        int IRdown = Irreps::directProd(IR,Btensors[theindex+1][l_beta-l_alpha][theindex+1-l_beta]->get_irrep());
                         int memSkappa = denS->gKappa(NL+2,TwoSLdown,ILdown,N1,N2,TwoJ,NR+2,TwoSRdown,IRdown);
                
                         if (memSkappa!=-1){
@@ -430,7 +430,7 @@ void CheMPS2::Heff::addDiagram2a2spin1(const int ikappa, double * memS, double *
             if ((TwoSLdown>=0) && (TwoSRdown>=0) && (abs(TwoSLdown-TwoSRdown)<=TwoJ)){
             
                int fase = phase(TwoSLdown+TwoSR+TwoJ+2);
-               const double thefactor = fase * sqrt((TwoSRdown + 1)*(TwoSLdown + 1.0)) * gsl_sf_coupling_6j(TwoSLdown,TwoSRdown,TwoJ,TwoSR,TwoSL,2);
+               const double thefactor = fase * sqrt((TwoSRdown + 1)*(TwoSLdown + 1.0)) * Wigner::wigner6j(TwoSLdown,TwoSRdown,TwoJ,TwoSR,TwoSL,2);
       
                for (int l_gamma=theindex+2; l_gamma<Prob->gL(); l_gamma++){
                   for (int l_delta=l_gamma+1; l_delta<Prob->gL(); l_delta++){
@@ -439,8 +439,8 @@ void CheMPS2::Heff::addDiagram2a2spin1(const int ikappa, double * memS, double *
                      if ( MPIchemps2::owner_absigma( l_gamma, l_delta ) == MPIRANK )
                      #endif
                      {
-                        int ILdown = Irreps::directProd(IL,Btensors[theindex-1][l_delta-l_gamma][l_gamma-theindex]->gIdiff());
-                        int IRdown = Irreps::directProd(IR,S1tensors[theindex+1][l_delta-l_gamma][l_gamma-theindex-2]->gIdiff());
+                        int ILdown = Irreps::directProd(IL,Btensors[theindex-1][l_delta-l_gamma][l_gamma-theindex]->get_irrep());
+                        int IRdown = Irreps::directProd(IR,S1tensors[theindex+1][l_delta-l_gamma][l_gamma-theindex-2]->get_irrep());
                         int memSkappa = denS->gKappa(NL+2,TwoSLdown,ILdown,N1,N2,TwoJ,NR+2,TwoSRdown,IRdown);
                 
                         if (memSkappa!=-1){
@@ -473,7 +473,7 @@ void CheMPS2::Heff::addDiagram2a2spin1(const int ikappa, double * memS, double *
    
 }
 
-void CheMPS2::Heff::addDiagram2a3spin0(const int ikappa, double * memS, double * memHeff, const Sobject * denS, TensorC **** Ctensors, TensorF0 **** F0tensors, double * workspace) const{
+void CheMPS2::Heff::addDiagram2a3spin0(const int ikappa, double * memS, double * memHeff, const Sobject * denS, TensorOperator **** Ctensors, TensorF0 **** F0tensors, double * workspace) const{
 
    #ifdef CHEMPS2_MPI_COMPILATION
    const int MPIRANK = MPIchemps2::mpi_rank();
@@ -506,8 +506,8 @@ void CheMPS2::Heff::addDiagram2a3spin0(const int ikappa, double * memS, double *
             if ( MPIchemps2::owner_cdf( Prob->gL(), l_gamma, l_alpha ) == MPIRANK )
             #endif
             {
-               int ILdown = Irreps::directProd(IL,F0tensors[theindex-1][l_alpha-l_gamma][theindex-1-l_alpha]->gIdiff());
-               int IRdown = Irreps::directProd(IR,Ctensors[theindex+1][l_alpha-l_gamma][theindex+1-l_alpha]->gIdiff());
+               int ILdown = Irreps::directProd(IL,F0tensors[theindex-1][l_alpha-l_gamma][theindex-1-l_alpha]->get_irrep());
+               int IRdown = Irreps::directProd(IR,Ctensors[theindex+1][l_alpha-l_gamma][theindex+1-l_alpha]->get_irrep());
                int memSkappa = denS->gKappa(NL,TwoSL,ILdown,N1,N2,TwoJ,NR,TwoSR,IRdown);
                
                if (memSkappa!=-1){
@@ -542,8 +542,8 @@ void CheMPS2::Heff::addDiagram2a3spin0(const int ikappa, double * memS, double *
             if ( MPIchemps2::owner_cdf( Prob->gL(), l_alpha, l_gamma ) == MPIRANK )
             #endif
             {
-               int ILdown = Irreps::directProd(IL,F0tensors[theindex-1][l_gamma-l_alpha][theindex-1-l_gamma]->gIdiff());
-               int IRdown = Irreps::directProd(IR,Ctensors[theindex+1][l_gamma-l_alpha][theindex+1-l_gamma]->gIdiff());
+               int ILdown = Irreps::directProd(IL,F0tensors[theindex-1][l_gamma-l_alpha][theindex-1-l_gamma]->get_irrep());
+               int IRdown = Irreps::directProd(IR,Ctensors[theindex+1][l_gamma-l_alpha][theindex+1-l_gamma]->get_irrep());
                int memSkappa = denS->gKappa(NL,TwoSL,ILdown,N1,N2,TwoJ,NR,TwoSR,IRdown);
                
                if (memSkappa!=-1){
@@ -580,8 +580,8 @@ void CheMPS2::Heff::addDiagram2a3spin0(const int ikappa, double * memS, double *
             if ( MPIchemps2::owner_cdf( Prob->gL(), l_delta, l_beta ) == MPIRANK )
             #endif
             {
-               int ILdown = Irreps::directProd(IL,Ctensors[theindex-1][l_beta-l_delta][l_delta-theindex]->gIdiff());
-               int IRdown = Irreps::directProd(IR,F0tensors[theindex+1][l_beta-l_delta][l_delta-theindex-2]->gIdiff());
+               int ILdown = Irreps::directProd(IL,Ctensors[theindex-1][l_beta-l_delta][l_delta-theindex]->get_irrep());
+               int IRdown = Irreps::directProd(IR,F0tensors[theindex+1][l_beta-l_delta][l_delta-theindex-2]->get_irrep());
                int memSkappa = denS->gKappa(NL,TwoSL,ILdown,N1,N2,TwoJ,NR,TwoSR,IRdown);
                
                if (memSkappa!=-1){
@@ -616,8 +616,8 @@ void CheMPS2::Heff::addDiagram2a3spin0(const int ikappa, double * memS, double *
             if ( MPIchemps2::owner_cdf( Prob->gL(), l_beta, l_delta ) == MPIRANK )
             #endif
             {
-               int ILdown = Irreps::directProd(IL,Ctensors[theindex-1][l_delta-l_beta][l_beta-theindex]->gIdiff());
-               int IRdown = Irreps::directProd(IR,F0tensors[theindex+1][l_delta-l_beta][l_beta-theindex-2]->gIdiff());
+               int ILdown = Irreps::directProd(IL,Ctensors[theindex-1][l_delta-l_beta][l_beta-theindex]->get_irrep());
+               int IRdown = Irreps::directProd(IR,F0tensors[theindex+1][l_delta-l_beta][l_beta-theindex-2]->get_irrep());
                int memSkappa = denS->gKappa(NL,TwoSL,ILdown,N1,N2,TwoJ,NR,TwoSR,IRdown);
                
                if (memSkappa!=-1){
@@ -648,7 +648,7 @@ void CheMPS2::Heff::addDiagram2a3spin0(const int ikappa, double * memS, double *
    
 }
 
-void CheMPS2::Heff::addDiagram2a3spin1(const int ikappa, double * memS, double * memHeff, const Sobject * denS, TensorD **** Dtensors, TensorF1 **** F1tensors, double * workspace) const{
+void CheMPS2::Heff::addDiagram2a3spin1(const int ikappa, double * memS, double * memHeff, const Sobject * denS, TensorOperator **** Dtensors, TensorF1 **** F1tensors, double * workspace) const{
 
    #ifdef CHEMPS2_MPI_COMPILATION
    const int MPIRANK = MPIchemps2::mpi_rank();
@@ -680,7 +680,7 @@ void CheMPS2::Heff::addDiagram2a3spin1(const int ikappa, double * memS, double *
             if ((TwoSLdown>=0) && (TwoSRdown>=0) && (abs(TwoSLdown-TwoSRdown)<=TwoJ)){
             
                int fase = phase(TwoSLdown+TwoSRdown+TwoJ+2);
-               double prefactor = fase * sqrt((TwoSR + 1)*(TwoSLdown + 1.0)) * gsl_sf_coupling_6j(TwoSLdown,TwoSRdown,TwoJ,TwoSR,TwoSL,2);
+               double prefactor = fase * sqrt((TwoSR + 1)*(TwoSLdown + 1.0)) * Wigner::wigner6j(TwoSLdown,TwoSRdown,TwoJ,TwoSR,TwoSL,2);
       
                for (int l_gamma=0; l_gamma<theindex; l_gamma++){
                   for (int l_alpha=l_gamma+1; l_alpha<theindex; l_alpha++){
@@ -689,8 +689,8 @@ void CheMPS2::Heff::addDiagram2a3spin1(const int ikappa, double * memS, double *
                      if ( MPIchemps2::owner_cdf( Prob->gL(), l_gamma, l_alpha ) == MPIRANK )
                      #endif
                      {
-                        int ILdown = Irreps::directProd(IL,F1tensors[theindex-1][l_alpha-l_gamma][theindex-1-l_alpha]->gIdiff());
-                        int IRdown = Irreps::directProd(IR,Dtensors[theindex+1][l_alpha-l_gamma][theindex+1-l_alpha]->gIdiff());
+                        int ILdown = Irreps::directProd(IL,F1tensors[theindex-1][l_alpha-l_gamma][theindex-1-l_alpha]->get_irrep());
+                        int IRdown = Irreps::directProd(IR,Dtensors[theindex+1][l_alpha-l_gamma][theindex+1-l_alpha]->get_irrep());
                         int memSkappa = denS->gKappa(NL,TwoSLdown,ILdown,N1,N2,TwoJ,NR,TwoSRdown,IRdown);
                
                         if (memSkappa!=-1){
@@ -718,7 +718,7 @@ void CheMPS2::Heff::addDiagram2a3spin1(const int ikappa, double * memS, double *
                }
                
                fase = phase(TwoSL+TwoSR+TwoJ+2);
-               prefactor = fase * sqrt((TwoSRdown + 1)*(TwoSL + 1.0)) * gsl_sf_coupling_6j(TwoSLdown,TwoSRdown,TwoJ,TwoSR,TwoSL,2);
+               prefactor = fase * sqrt((TwoSRdown + 1)*(TwoSL + 1.0)) * Wigner::wigner6j(TwoSLdown,TwoSRdown,TwoJ,TwoSR,TwoSL,2);
       
                for (int l_alpha=0; l_alpha<theindex; l_alpha++){
                   for (int l_gamma=l_alpha; l_gamma<theindex; l_gamma++){
@@ -727,8 +727,8 @@ void CheMPS2::Heff::addDiagram2a3spin1(const int ikappa, double * memS, double *
                      if ( MPIchemps2::owner_cdf( Prob->gL(), l_alpha, l_gamma ) == MPIRANK )
                      #endif
                      {
-                        int ILdown = Irreps::directProd(IL,F1tensors[theindex-1][l_gamma-l_alpha][theindex-1-l_gamma]->gIdiff());
-                        int IRdown = Irreps::directProd(IR,Dtensors[theindex+1][l_gamma-l_alpha][theindex+1-l_gamma]->gIdiff());
+                        int ILdown = Irreps::directProd(IL,F1tensors[theindex-1][l_gamma-l_alpha][theindex-1-l_gamma]->get_irrep());
+                        int IRdown = Irreps::directProd(IR,Dtensors[theindex+1][l_gamma-l_alpha][theindex+1-l_gamma]->get_irrep());
                         int memSkappa = denS->gKappa(NL,TwoSLdown,ILdown,N1,N2,TwoJ,NR,TwoSRdown,IRdown);
                
                         if (memSkappa!=-1){
@@ -766,7 +766,7 @@ void CheMPS2::Heff::addDiagram2a3spin1(const int ikappa, double * memS, double *
             if ((TwoSLdown>=0) && (TwoSRdown>=0) && (abs(TwoSLdown-TwoSRdown)<=TwoJ)){
             
                int fase = phase(TwoSLdown+TwoSRdown+TwoJ+2);
-               double prefactor = fase * sqrt((TwoSR + 1)*(TwoSLdown + 1.0)) * gsl_sf_coupling_6j(TwoSLdown,TwoSRdown,TwoJ,TwoSR,TwoSL,2);
+               double prefactor = fase * sqrt((TwoSR + 1)*(TwoSLdown + 1.0)) * Wigner::wigner6j(TwoSLdown,TwoSRdown,TwoJ,TwoSR,TwoSL,2);
       
                for (int l_delta=theindex+2; l_delta<Prob->gL(); l_delta++){
                   for (int l_beta=l_delta+1; l_beta<Prob->gL(); l_beta++){
@@ -775,8 +775,8 @@ void CheMPS2::Heff::addDiagram2a3spin1(const int ikappa, double * memS, double *
                      if ( MPIchemps2::owner_cdf( Prob->gL(), l_delta, l_beta ) == MPIRANK )
                      #endif
                      {
-                        int ILdown = Irreps::directProd(IL,Dtensors[theindex-1][l_beta-l_delta][l_delta-theindex]->gIdiff());
-                        int IRdown = Irreps::directProd(IR,F1tensors[theindex+1][l_beta-l_delta][l_delta-theindex-2]->gIdiff());
+                        int ILdown = Irreps::directProd(IL,Dtensors[theindex-1][l_beta-l_delta][l_delta-theindex]->get_irrep());
+                        int IRdown = Irreps::directProd(IR,F1tensors[theindex+1][l_beta-l_delta][l_delta-theindex-2]->get_irrep());
                         int memSkappa = denS->gKappa(NL,TwoSLdown,ILdown,N1,N2,TwoJ,NR,TwoSRdown,IRdown);
                
                         if (memSkappa!=-1){
@@ -804,7 +804,7 @@ void CheMPS2::Heff::addDiagram2a3spin1(const int ikappa, double * memS, double *
                }
                
                fase = phase(TwoSL+TwoSR+TwoJ+2);
-               prefactor = fase * sqrt((TwoSRdown + 1)*(TwoSL + 1.0)) * gsl_sf_coupling_6j(TwoSLdown,TwoSRdown,TwoJ,TwoSR,TwoSL,2);
+               prefactor = fase * sqrt((TwoSRdown + 1)*(TwoSL + 1.0)) * Wigner::wigner6j(TwoSLdown,TwoSRdown,TwoJ,TwoSR,TwoSL,2);
       
                for (int l_beta=theindex+2; l_beta<Prob->gL(); l_beta++){
                   for (int l_delta=l_beta; l_delta<Prob->gL(); l_delta++){
@@ -813,8 +813,8 @@ void CheMPS2::Heff::addDiagram2a3spin1(const int ikappa, double * memS, double *
                      if ( MPIchemps2::owner_cdf( Prob->gL(), l_beta, l_delta ) == MPIRANK )
                      #endif
                      {
-                        int ILdown = Irreps::directProd(IL,Dtensors[theindex-1][l_delta-l_beta][l_beta-theindex]->gIdiff());
-                        int IRdown = Irreps::directProd(IR,F1tensors[theindex+1][l_delta-l_beta][l_beta-theindex-2]->gIdiff());
+                        int ILdown = Irreps::directProd(IL,Dtensors[theindex-1][l_delta-l_beta][l_beta-theindex]->get_irrep());
+                        int IRdown = Irreps::directProd(IR,F1tensors[theindex+1][l_delta-l_beta][l_beta-theindex-2]->get_irrep());
                         int memSkappa = denS->gKappa(NL,TwoSLdown,ILdown,N1,N2,TwoJ,NR,TwoSRdown,IRdown);
                         
                         if (memSkappa!=-1){
@@ -847,7 +847,7 @@ void CheMPS2::Heff::addDiagram2a3spin1(const int ikappa, double * memS, double *
    
 }
 
-void CheMPS2::Heff::addDiagram2b1and2b2(const int ikappa, double * memS, double * memHeff, const Sobject * denS, TensorA * Atensor) const{
+void CheMPS2::Heff::addDiagram2b1and2b2(const int ikappa, double * memS, double * memHeff, const Sobject * denS, TensorOperator * Atensor) const{
 
    int N1 = denS->gN1(ikappa);
    
@@ -920,7 +920,7 @@ void CheMPS2::Heff::addDiagram2b1and2b2(const int ikappa, double * memS, double
    
 }
 
-void CheMPS2::Heff::addDiagram2c1and2c2(const int ikappa, double * memS, double * memHeff, const Sobject * denS, TensorA * Atensor) const{
+void CheMPS2::Heff::addDiagram2c1and2c2(const int ikappa, double * memS, double * memHeff, const Sobject * denS, TensorOperator * Atensor) const{
 
    int N2 = denS->gN2(ikappa);
    
@@ -1057,7 +1057,7 @@ void CheMPS2::Heff::addDiagram2dall(const int ikappa, double * memS, double * me
    
 }
 
-void CheMPS2::Heff::addDiagram2e1and2e2(const int ikappa, double * memS, double * memHeff, const Sobject * denS, TensorA * Atensor) const{
+void CheMPS2::Heff::addDiagram2e1and2e2(const int ikappa, double * memS, double * memHeff, const Sobject * denS, TensorOperator * Atensor) const{
 
    int N1 = denS->gN1(ikappa);
    
@@ -1120,7 +1120,7 @@ void CheMPS2::Heff::addDiagram2e1and2e2(const int ikappa, double * memS, double
    
 }
 
-void CheMPS2::Heff::addDiagram2f1and2f2(const int ikappa, double * memS, double * memHeff, const Sobject * denS, TensorA * Atensor) const{
+void CheMPS2::Heff::addDiagram2f1and2f2(const int ikappa, double * memS, double * memHeff, const Sobject * denS, TensorOperator * Atensor) const{
 
    int N2 = denS->gN2(ikappa);
    
@@ -1183,7 +1183,7 @@ void CheMPS2::Heff::addDiagram2f1and2f2(const int ikappa, double * memS, double
    
 }
 
-void CheMPS2::Heff::addDiagram2b3spin0(const int ikappa, double * memS, double * memHeff, const Sobject * denS, TensorC * Ctensor) const{
+void CheMPS2::Heff::addDiagram2b3spin0(const int ikappa, double * memS, double * memHeff, const Sobject * denS, TensorOperator * Ctensor) const{
 
    int N1 = denS->gN1(ikappa);
    
@@ -1211,7 +1211,7 @@ void CheMPS2::Heff::addDiagram2b3spin0(const int ikappa, double * memS, double *
    
 }
 
-void CheMPS2::Heff::addDiagram2c3spin0(const int ikappa, double * memS, double * memHeff, const Sobject * denS, TensorC * Ctensor) const{
+void CheMPS2::Heff::addDiagram2c3spin0(const int ikappa, double * memS, double * memHeff, const Sobject * denS, TensorOperator * Ctensor) const{
 
    int N2 = denS->gN2(ikappa);
    
@@ -1239,7 +1239,7 @@ void CheMPS2::Heff::addDiagram2c3spin0(const int ikappa, double * memS, double *
    
 }
 
-void CheMPS2::Heff::addDiagram2e3spin0(const int ikappa, double * memS, double * memHeff, const Sobject * denS, TensorC * Ctensor) const{
+void CheMPS2::Heff::addDiagram2e3spin0(const int ikappa, double * memS, double * memHeff, const Sobject * denS, TensorOperator * Ctensor) const{
 
    int N1 = denS->gN1(ikappa);
    
@@ -1266,7 +1266,7 @@ void CheMPS2::Heff::addDiagram2e3spin0(const int ikappa, double * memS, double *
    
 }
 
-void CheMPS2::Heff::addDiagram2f3spin0(const int ikappa, double * memS, double * memHeff, const Sobject * denS, TensorC * Ctensor) const{
+void CheMPS2::Heff::addDiagram2f3spin0(const int ikappa, double * memS, double * memHeff, const Sobject * denS, TensorOperator * Ctensor) const{
 
    int N2 = denS->gN2(ikappa);
    
@@ -1293,7 +1293,7 @@ void CheMPS2::Heff::addDiagram2f3spin0(const int ikappa, double * memS, double *
    
 }
 
-void CheMPS2::Heff::addDiagram2b3spin1(const int ikappa, double * memS, double * memHeff, const Sobject * denS, TensorD * Dtensor) const{
+void CheMPS2::Heff::addDiagram2b3spin1(const int ikappa, double * memS, double * memHeff, const Sobject * denS, TensorOperator * Dtensor) const{
 
    int N1 = denS->gN1(ikappa);
    
@@ -1331,7 +1331,7 @@ void CheMPS2::Heff::addDiagram2b3spin1(const int ikappa, double * memS, double *
                   if (memSkappa!=-1){
             
                      int fase = phase(TwoSLdown + TwoSR + TwoJ + TwoS2 + TwoJdown - 1);
-                     double alpha = fase * sqrt(3.0*(TwoJ+1)*(TwoJdown+1)*(TwoSL+1)) * gsl_sf_coupling_6j(TwoJdown,TwoJ,2,1,1,TwoS2) * gsl_sf_coupling_6j(TwoJdown,TwoJ,2,TwoSL,TwoSLdown,TwoSR);
+                     double alpha = fase * sqrt(3.0*(TwoJ+1)*(TwoJdown+1)*(TwoSL+1)) * Wigner::wigner6j(TwoJdown,TwoJ,2,1,1,TwoS2) * Wigner::wigner6j(TwoJdown,TwoJ,2,TwoSL,TwoSLdown,TwoSR);
                      char trans = 'T';
                      char notra = 'N';
                      double beta = 1.0;
@@ -1347,7 +1347,7 @@ void CheMPS2::Heff::addDiagram2b3spin1(const int ikappa, double * memS, double *
    
 }
 
-void CheMPS2::Heff::addDiagram2c3spin1(const int ikappa, double * memS, double * memHeff, const Sobject * denS, TensorD * Dtensor) const{
+void CheMPS2::Heff::addDiagram2c3spin1(const int ikappa, double * memS, double * memHeff, const Sobject * denS, TensorOperator * Dtensor) const{
 
    int N2 = denS->gN2(ikappa);
    
@@ -1385,7 +1385,7 @@ void CheMPS2::Heff::addDiagram2c3spin1(const int ikappa, double * memS, double *
                   if (memSkappa!=-1){
             
                      int fase = phase(TwoSLdown + TwoSR + 2*TwoJ + TwoS1 - 1);
-                     double alpha = fase * sqrt(3.0*(TwoJ+1)*(TwoJdown+1)*(TwoSL+1)) * gsl_sf_coupling_6j(TwoJdown,TwoJ,2,1,1,TwoS1) * gsl_sf_coupling_6j(TwoJdown,TwoJ,2,TwoSL,TwoSLdown,TwoSR);
+                     double alpha = fase * sqrt(3.0*(TwoJ+1)*(TwoJdown+1)*(TwoSL+1)) * Wigner::wigner6j(TwoJdown,TwoJ,2,1,1,TwoS1) * Wigner::wigner6j(TwoJdown,TwoJ,2,TwoSL,TwoSLdown,TwoSR);
                      char trans = 'T';
                      char notra = 'N';
                      double beta = 1.0;
@@ -1401,7 +1401,7 @@ void CheMPS2::Heff::addDiagram2c3spin1(const int ikappa, double * memS, double *
    
 }
 
-void CheMPS2::Heff::addDiagram2e3spin1(const int ikappa, double * memS, double * memHeff, const Sobject * denS, TensorD * Dtensor) const{
+void CheMPS2::Heff::addDiagram2e3spin1(const int ikappa, double * memS, double * memHeff, const Sobject * denS, TensorOperator * Dtensor) const{
 
    int N1 = denS->gN1(ikappa);
    
@@ -1439,7 +1439,7 @@ void CheMPS2::Heff::addDiagram2e3spin1(const int ikappa, double * memS, double *
                   if (memSkappa!=-1){
             
                      int fase = phase(TwoSRdown + TwoSL + 2*TwoJ + TwoS2 + 1);
-                     double alpha = fase * sqrt(3.0*(TwoJ+1)*(TwoJdown+1)*(TwoSRdown+1)) * gsl_sf_coupling_6j(TwoJdown,TwoJ,2,1,1,TwoS2) * gsl_sf_coupling_6j(TwoJdown,TwoJ,2,TwoSR,TwoSRdown,TwoSL);
+                     double alpha = fase * sqrt(3.0*(TwoJ+1)*(TwoJdown+1)*(TwoSRdown+1)) * Wigner::wigner6j(TwoJdown,TwoJ,2,1,1,TwoS2) * Wigner::wigner6j(TwoJdown,TwoJ,2,TwoSR,TwoSRdown,TwoSL);
                      char notr = 'N';
                      double beta = 1.0;
                
@@ -1454,7 +1454,7 @@ void CheMPS2::Heff::addDiagram2e3spin1(const int ikappa, double * memS, double *
    
 }
 
-void CheMPS2::Heff::addDiagram2f3spin1(const int ikappa, double * memS, double * memHeff, const Sobject * denS, TensorD * Dtensor) const{
+void CheMPS2::Heff::addDiagram2f3spin1(const int ikappa, double * memS, double * memHeff, const Sobject * denS, TensorOperator * Dtensor) const{
 
    int N2 = denS->gN2(ikappa);
    
@@ -1492,7 +1492,7 @@ void CheMPS2::Heff::addDiagram2f3spin1(const int ikappa, double * memS, double *
                   if (memSkappa!=-1){
             
                      int fase = phase(TwoSRdown + TwoSL + TwoJ + TwoS1 + TwoJdown + 1);
-                     double alpha = fase * sqrt(3.0*(TwoJ+1)*(TwoJdown+1)*(TwoSRdown+1)) * gsl_sf_coupling_6j(TwoJdown,TwoJ,2,1,1,TwoS1) * gsl_sf_coupling_6j(TwoJdown,TwoJ,2,TwoSR,TwoSRdown,TwoSL);
+                     double alpha = fase * sqrt(3.0*(TwoJ+1)*(TwoJdown+1)*(TwoSRdown+1)) * Wigner::wigner6j(TwoJdown,TwoJ,2,1,1,TwoS1) * Wigner::wigner6j(TwoJdown,TwoJ,2,TwoSR,TwoSRdown,TwoSL);
                      char notr = 'N';
                      double beta = 1.0;
                
