@@ -1,6 +1,6 @@
 /*
    CheMPS2: a spin-adapted implementation of DMRG for ab initio quantum chemistry
-   Copyright (C) 2013-2015 Sebastian Wouters
+   Copyright (C) 2013-2016 Sebastian Wouters
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -22,8 +22,8 @@
 
 #include "Heff.h"
 #include "Lapack.h"
-#include "Gsl.h"
 #include "MPIchemps2.h"
+#include "Wigner.h"
 
 void CheMPS2::Heff::addDiagram3Aand3D(const int ikappa, double * memS, double * memHeff, const Sobject * denS, TensorQ * Qleft, TensorL ** Lleft, double * temp) const{
 
@@ -57,7 +57,7 @@ void CheMPS2::Heff::addDiagram3Aand3D(const int ikappa, double * memS, double *
                   int memSkappa = denS->gKappa(NL+1,TwoSLdown,ILdown,1,N2,TwoJdown,NR,TwoSR,IR);
                   if (memSkappa!=-1){
                      int fase = phase(TwoSL+TwoSR+2+TwoS2);
-                     double factor = sqrt((TwoJdown+1)*(TwoSLdown+1.0))*fase*gsl_sf_coupling_6j(TwoJdown,TwoS2,1,TwoSL,TwoSLdown,TwoSR);
+                     double factor = sqrt((TwoJdown+1)*(TwoSLdown+1.0))*fase*Wigner::wigner6j(TwoJdown,TwoS2,1,TwoSL,TwoSLdown,TwoSR);
                      double beta = 1.0; //add
                      char notr = 'N';
                   
@@ -89,7 +89,7 @@ void CheMPS2::Heff::addDiagram3Aand3D(const int ikappa, double * memS, double *
             int memSkappa = denS->gKappa(NL+1,TwoSLdown,ILdown,0,N2,TwoS2,NR,TwoSR,IR);
             if (memSkappa!=-1){
                int fase = phase(TwoSL+TwoSR+1+TwoS2);
-               double factor = sqrt((TwoSLdown+1)*(TwoJ+1.0))*fase*gsl_sf_coupling_6j(TwoS2,TwoJ,1,TwoSL,TwoSLdown,TwoSR);
+               double factor = sqrt((TwoSLdown+1)*(TwoJ+1.0))*fase*Wigner::wigner6j(TwoS2,TwoJ,1,TwoSL,TwoSLdown,TwoSR);
                double beta = 1.0;
                char notr = 'N';
                double * BlockQ = Qleft->gStorage(NL,TwoSL,IL,NL+1,TwoSLdown,ILdown);
@@ -112,7 +112,7 @@ void CheMPS2::Heff::addDiagram3Aand3D(const int ikappa, double * memS, double *
                   int memSkappa = denS->gKappa(NL-1,TwoSLdown,ILdown,1,N2,TwoJdown,NR,TwoSR,IR);
                   if (memSkappa!=-1){
                      int fase = phase(TwoSLdown+TwoSR+1+TwoS2);
-                     double factor = fase*sqrt((TwoSL+1)*(TwoJdown+1.0))*gsl_sf_coupling_6j(TwoJdown,TwoS2,1,TwoSL,TwoSLdown,TwoSR);
+                     double factor = fase*sqrt((TwoSL+1)*(TwoJdown+1.0))*Wigner::wigner6j(TwoJdown,TwoS2,1,TwoSL,TwoSLdown,TwoSR);
                      double beta = 1.0;
                      char notr = 'N';
                      char trans = 'T';
@@ -132,7 +132,7 @@ void CheMPS2::Heff::addDiagram3Aand3D(const int ikappa, double * memS, double *
             int memSkappa = denS->gKappa(NL-1,TwoSLdown,ILdown,2,N2,TwoS2,NR,TwoSR,IR);
             if (memSkappa!=-1){
                int fase = phase(TwoSLdown+TwoSR+2+TwoS2);
-               double factor = fase*sqrt((TwoSL+1)*(TwoJ+1.0))*gsl_sf_coupling_6j(TwoS2,TwoJ,1,TwoSL,TwoSLdown,TwoSR);
+               double factor = fase*sqrt((TwoSL+1)*(TwoJ+1.0))*Wigner::wigner6j(TwoS2,TwoJ,1,TwoSL,TwoSLdown,TwoSR);
                double beta = 1.0;
                char notr = 'N';
                char trans = 'T';
@@ -190,7 +190,7 @@ void CheMPS2::Heff::addDiagram3Band3I(const int ikappa, double * memS, double *
                   int memSkappa = denS->gKappa(NL+1,TwoSLdown,ILdown,N1,1,TwoJdown,NR,TwoSR,IR);
                   if (memSkappa!=-1){
                      int fase = phase(TwoSL+TwoSR+3-TwoJdown);
-                     double factor = sqrt((TwoJdown+1)*(TwoSLdown+1.0))*fase*gsl_sf_coupling_6j(TwoJdown,TwoS1,1,TwoSL,TwoSLdown,TwoSR);
+                     double factor = sqrt((TwoJdown+1)*(TwoSLdown+1.0))*fase*Wigner::wigner6j(TwoJdown,TwoS1,1,TwoSL,TwoSLdown,TwoSR);
                      double beta = 1.0; //add
                      char notr = 'N';
                   
@@ -222,7 +222,7 @@ void CheMPS2::Heff::addDiagram3Band3I(const int ikappa, double * memS, double *
             int memSkappa = denS->gKappa(NL+1,TwoSLdown,ILdown,N1,0,TwoS1,NR,TwoSR,IR);
             if (memSkappa!=-1){
                int fase = phase(TwoSL+TwoSR+2-TwoJ);
-               double factor = sqrt((TwoSLdown+1)*(TwoJ+1.0))*fase*gsl_sf_coupling_6j(TwoS1,TwoJ,1,TwoSL,TwoSLdown,TwoSR);
+               double factor = sqrt((TwoSLdown+1)*(TwoJ+1.0))*fase*Wigner::wigner6j(TwoS1,TwoJ,1,TwoSL,TwoSLdown,TwoSR);
                double beta = 1.0;
                char notr = 'N';
                double * BlockQ = Qleft->gStorage(NL,TwoSL,IL,NL+1,TwoSLdown,ILdown);
@@ -245,7 +245,7 @@ void CheMPS2::Heff::addDiagram3Band3I(const int ikappa, double * memS, double *
                   int memSkappa = denS->gKappa(NL-1,TwoSLdown,ILdown,N1,1,TwoJdown,NR,TwoSR,IR);
                   if (memSkappa!=-1){
                      int fase = phase(TwoSLdown+TwoSR+2-TwoJdown);
-                     double factor = fase*sqrt((TwoSL+1)*(TwoJdown+1.0))*gsl_sf_coupling_6j(TwoJdown,TwoS1,1,TwoSL,TwoSLdown,TwoSR);
+                     double factor = fase*sqrt((TwoSL+1)*(TwoJdown+1.0))*Wigner::wigner6j(TwoJdown,TwoS1,1,TwoSL,TwoSLdown,TwoSR);
                      double beta = 1.0;
                      char notr = 'N';
                      char trans = 'T';
@@ -265,7 +265,7 @@ void CheMPS2::Heff::addDiagram3Band3I(const int ikappa, double * memS, double *
             int memSkappa = denS->gKappa(NL-1,TwoSLdown,ILdown,N1,2,TwoS1,NR,TwoSR,IR);
             if (memSkappa!=-1){
                int fase = phase(TwoSLdown+TwoSR+3-TwoJ);
-               double factor = fase*sqrt((TwoSL+1)*(TwoJ+1.0))*gsl_sf_coupling_6j(TwoS1,TwoJ,1,TwoSL,TwoSLdown,TwoSR);
+               double factor = fase*sqrt((TwoSL+1)*(TwoJ+1.0))*Wigner::wigner6j(TwoS1,TwoJ,1,TwoSL,TwoSLdown,TwoSR);
                double beta = 1.0;
                char notr = 'N';
                char trans = 'T';
@@ -318,7 +318,7 @@ void CheMPS2::Heff::addDiagram3C(const int ikappa, double * memS, double * memHe
          if ((abs(TwoSLdown-TwoSRdown)<=TwoJ) && (TwoSLdown>=0) && (TwoSRdown>=0)){
       
             int fase = phase(TwoSLdown+TwoSR+TwoJ+1 + ((N1==1)?2:0) + ((N2==1)?2:0) );
-            const double factor = fase * sqrt((TwoSLdown+1)*(TwoSRdown+1.0)) * gsl_sf_coupling_6j(TwoSL,TwoSR,TwoJ,TwoSRdown,TwoSLdown,1);
+            const double factor = fase * sqrt((TwoSLdown+1)*(TwoSRdown+1.0)) * Wigner::wigner6j(TwoSL,TwoSR,TwoJ,TwoSRdown,TwoSLdown,1);
       
             for (int l_index=theindex+2; l_index<Prob->gL(); l_index++){
             
@@ -359,7 +359,7 @@ void CheMPS2::Heff::addDiagram3C(const int ikappa, double * memS, double * memHe
          if ((abs(TwoSLdown-TwoSRdown)<=TwoJ) && (TwoSLdown>=0) && (TwoSRdown>=0)){
       
             int fase = phase(TwoSL+TwoSRdown+TwoJ+1 + ((N1==1)?2:0) + ((N2==1)?2:0) );
-            const double factor = fase * sqrt((TwoSL+1)*(TwoSR+1.0)) * gsl_sf_coupling_6j(TwoSL,TwoSR,TwoJ,TwoSRdown,TwoSLdown,1);
+            const double factor = fase * sqrt((TwoSL+1)*(TwoSR+1.0)) * Wigner::wigner6j(TwoSL,TwoSR,TwoJ,TwoSRdown,TwoSLdown,1);
       
             for (int l_index=theindex+2; l_index<Prob->gL(); l_index++){
             
@@ -499,7 +499,7 @@ void CheMPS2::Heff::addDiagram3Kand3F(const int ikappa, double * memS, double *
             int memSkappa = denS->gKappa(NL,TwoSL,IL,0,N2,TwoS2,NR-1,TwoSRdown,IRdown);
             if (memSkappa!=-1){
                int fase = phase(TwoSL+TwoSR+TwoJ+2*TwoS2);
-               double factor = sqrt((TwoJ+1)*(TwoSR+1.0)) * fase * gsl_sf_coupling_6j(TwoS2,TwoJ,1,TwoSR,TwoSRdown,TwoSL);
+               double factor = sqrt((TwoJ+1)*(TwoSR+1.0)) * fase * Wigner::wigner6j(TwoS2,TwoJ,1,TwoSR,TwoSRdown,TwoSL);
                double beta = 1.0; //add
                char notr = 'N';
                double * BlockQ = Qright->gStorage(NR-1,TwoSRdown,IRdown,NR,TwoSR,IR);
@@ -522,7 +522,7 @@ void CheMPS2::Heff::addDiagram3Kand3F(const int ikappa, double * memS, double *
                   int memSkappa = denS->gKappa(NL,TwoSL,IL,1,N2,TwoJdown,NR-1,TwoSRdown,IRdown);
                   if (memSkappa!=-1){
                      int fase = phase(TwoSL+TwoSR+TwoJdown+1+2*TwoS2);
-                     double factor = sqrt((TwoJdown+1)*(TwoSR+1.0)) * fase * gsl_sf_coupling_6j(TwoJdown,TwoS2,1,TwoSR,TwoSRdown,TwoSL);
+                     double factor = sqrt((TwoJdown+1)*(TwoSR+1.0)) * fase * Wigner::wigner6j( TwoJdown, TwoS2, 1, TwoSR, TwoSRdown, TwoSL );
                      double beta = 1.0; //add
                      char notr = 'N';
                   
@@ -560,7 +560,7 @@ void CheMPS2::Heff::addDiagram3Kand3F(const int ikappa, double * memS, double *
                   int memSkappa = denS->gKappa(NL,TwoSL,IL,1,N2,TwoJdown,NR+1,TwoSRdown,IRdown);
                   if (memSkappa!=-1){
                      int fase = phase(TwoSL+TwoSRdown+TwoJdown+2*TwoS2);
-                     double factor = sqrt((TwoJdown+1)*(TwoSRdown+1.0)) * fase * gsl_sf_coupling_6j(TwoJdown,TwoS2,1,TwoSR,TwoSRdown,TwoSL);
+                     double factor = sqrt((TwoJdown+1)*(TwoSRdown+1.0)) * fase * Wigner::wigner6j(TwoJdown,TwoS2,1,TwoSR,TwoSRdown,TwoSL);
                      double beta = 1.0; //add
                      char notr = 'N';
                      char tran = 'T';
@@ -580,7 +580,7 @@ void CheMPS2::Heff::addDiagram3Kand3F(const int ikappa, double * memS, double *
             int memSkappa = denS->gKappa(NL,TwoSL,IL,2,N2,TwoS2,NR+1,TwoSRdown,IRdown);
             if (memSkappa!=-1){
                int fase = phase(TwoSL+TwoSRdown+TwoJ+1+2*TwoS2);
-               double factor = sqrt((TwoJ+1)*(TwoSRdown+1.0)) * fase * gsl_sf_coupling_6j(TwoS2,TwoJ,1,TwoSR,TwoSRdown,TwoSL);
+               double factor = sqrt((TwoJ+1)*(TwoSRdown+1.0)) * fase * Wigner::wigner6j(TwoS2,TwoJ,1,TwoSR,TwoSRdown,TwoSL);
                double beta = 1.0; //add
                char notr = 'N';
                char tran = 'T';
@@ -632,7 +632,7 @@ void CheMPS2::Heff::addDiagram3Land3G(const int ikappa, double * memS, double *
             int memSkappa = denS->gKappa(NL,TwoSL,IL,N1,0,TwoS1,NR-1,TwoSRdown,IRdown);
             if (memSkappa!=-1){
                int fase = phase(TwoSL+TwoSR+TwoS1+1);
-               double factor = sqrt((TwoJ+1)*(TwoSR+1.0)) * fase * gsl_sf_coupling_6j(TwoS1,TwoJ,1,TwoSR,TwoSRdown,TwoSL);
+               double factor = sqrt((TwoJ+1)*(TwoSR+1.0)) * fase * Wigner::wigner6j(TwoS1,TwoJ,1,TwoSR,TwoSRdown,TwoSL);
                double beta = 1.0; //add
                char notr = 'N';
                double * BlockQ = Qright->gStorage(NR-1,TwoSRdown,IRdown,NR,TwoSR,IR);
@@ -655,7 +655,7 @@ void CheMPS2::Heff::addDiagram3Land3G(const int ikappa, double * memS, double *
                   int memSkappa = denS->gKappa(NL,TwoSL,IL,N1,1,TwoJdown,NR-1,TwoSRdown,IRdown);
                   if (memSkappa!=-1){
                      int fase = phase(TwoSL+TwoSR+TwoS1+2);
-                     double factor = sqrt((TwoJdown+1)*(TwoSR+1.0)) * fase * gsl_sf_coupling_6j(TwoJdown,TwoS1,1,TwoSR,TwoSRdown,TwoSL);
+                     double factor = sqrt((TwoJdown+1)*(TwoSR+1.0)) * fase * Wigner::wigner6j(TwoJdown,TwoS1,1,TwoSR,TwoSRdown,TwoSL);
                      double beta = 1.0; //add
                      char notr = 'N';
                   
@@ -693,7 +693,7 @@ void CheMPS2::Heff::addDiagram3Land3G(const int ikappa, double * memS, double *
                   int memSkappa = denS->gKappa(NL,TwoSL,IL,N1,1,TwoJdown,NR+1,TwoSRdown,IRdown);
                   if (memSkappa!=-1){
                      int fase = phase(TwoSL+TwoSRdown+TwoS1+1);
-                     double factor = sqrt((TwoJdown+1)*(TwoSRdown+1.0)) * fase * gsl_sf_coupling_6j(TwoJdown,TwoS1,1,TwoSR,TwoSRdown,TwoSL);
+                     double factor = sqrt((TwoJdown+1)*(TwoSRdown+1.0)) * fase * Wigner::wigner6j(TwoJdown,TwoS1,1,TwoSR,TwoSRdown,TwoSL);
                      double beta = 1.0; //add
                      char notr = 'N';
                      char tran = 'T';
@@ -713,7 +713,7 @@ void CheMPS2::Heff::addDiagram3Land3G(const int ikappa, double * memS, double *
             int memSkappa = denS->gKappa(NL,TwoSL,IL,N1,2,TwoS1,NR+1,TwoSRdown,IRdown);
             if (memSkappa!=-1){
                int fase = phase(TwoSL+TwoSRdown+TwoS1+2);
-               double factor = sqrt((TwoJ+1)*(TwoSRdown+1.0)) * fase * gsl_sf_coupling_6j(TwoS1,TwoJ,1,TwoSR,TwoSRdown,TwoSL);
+               double factor = sqrt((TwoJ+1)*(TwoSRdown+1.0)) * fase * Wigner::wigner6j(TwoS1,TwoJ,1,TwoSR,TwoSRdown,TwoSL);
                double beta = 1.0; //add
                char notr = 'N';
                char tran = 'T';
@@ -766,7 +766,7 @@ void CheMPS2::Heff::addDiagram3J(const int ikappa, double * memS, double * memHe
          if ((abs(TwoSLdown-TwoSRdown)<=TwoJ) && (TwoSLdown>=0) && (TwoSRdown>=0)){
       
             int fase = phase(TwoSLdown+TwoSR+TwoJ+1 + ((N1==1)?2:0) + ((N2==1)?2:0) );
-            const double factor = fase * sqrt((TwoSLdown+1)*(TwoSRdown+1.0)) * gsl_sf_coupling_6j(TwoSL,TwoSR,TwoJ,TwoSRdown,TwoSLdown,1);
+            const double factor = fase * sqrt((TwoSLdown+1)*(TwoSRdown+1.0)) * Wigner::wigner6j(TwoSL,TwoSR,TwoJ,TwoSRdown,TwoSLdown,1);
       
             for (int l_index=0; l_index<theindex; l_index++){
             
@@ -808,7 +808,7 @@ void CheMPS2::Heff::addDiagram3J(const int ikappa, double * memS, double * memHe
          if ((abs(TwoSLdown-TwoSRdown)<=TwoJ) && (TwoSLdown>=0) && (TwoSRdown>=0)){
       
             int fase = phase(TwoSL+TwoSRdown+TwoJ+1 + ((N1==1)?2:0) + ((N2==1)?2:0) );
-            const double factor = fase * sqrt((TwoSL+1)*(TwoSR+1.0)) * gsl_sf_coupling_6j(TwoSL,TwoSR,TwoJ,TwoSRdown,TwoSLdown,1);
+            const double factor = fase * sqrt((TwoSL+1)*(TwoSR+1.0)) * Wigner::wigner6j(TwoSL,TwoSR,TwoJ,TwoSRdown,TwoSLdown,1);
       
             for (int l_index=0; l_index<theindex; l_index++){
             
