@@ -333,9 +333,12 @@ cout << "\n"
 "       MOLCAS_FOCK = /path/to/fock/input\n"
 "              When all orbitals are active orbitals, read in this file containing the Fock operator (default unspecified).\n"
 "\n"
-"       MOLCAS_REORDER = bool\n"
+"       MOLCAS_FIEDLER = bool\n"
 "              When all orbitals are active orbitals, switch on orbital reordering based on the Fiedler vector of the exchange matrix (TRUE or FALSE; default FALSE).\n"
 "\n"
+"       MOLCAS_ORDER = int, int, int, int\n"
+"              When all orbitals are active orbitals, provide a custom orbital reordering (default unspecified). When specified, this option takes precedence over MOLCAS_FIEDLER.\n"
+"\n"
 "       MOLCAS_MPS = bool\n"
 "              When all orbitals are active orbitals, switch on the creation of MPS checkpoints (TRUE or FALSE; default FALSE).\n"
 "\n"
@@ -429,8 +432,9 @@ int main( int argc, char ** argv ){
    string molcas_3rdm    = "";
    string molcas_f4rdm   = "";
    string molcas_fock    = "";
-   bool   molcas_reorder = false;
+   bool   molcas_fiedler = false;
    bool   molcas_mps     = false;
+   string molcas_order   = "";
 
    bool   scf_state_avg    = false;
    double scf_diis_thr     = 0.0;
@@ -551,7 +555,7 @@ int main( int argc, char ** argv ){
       if ( find_character( &scf_active_space, line, "SCF_ACTIVE_SPACE", options1, 4 ) == false ){ return clean_exit( -1 ); }
       if ( find_character( &caspt2_orbs,      line, "CASPT2_ORBS",      options2, 2 ) == false ){ return clean_exit( -1 ); }
 
-      if ( find_boolean( &molcas_reorder, line, "MOLCAS_REORDER" ) == false ){ return clean_exit( -1 ); }
+      if ( find_boolean( &molcas_fiedler, line, "MOLCAS_FIEDLER" ) == false ){ return clean_exit( -1 ); }
       if ( find_boolean( &molcas_mps,     line, "MOLCAS_MPS"     ) == false ){ return clean_exit( -1 ); }
       if ( find_boolean( &scf_state_avg,  line, "SCF_STATE_AVG"  ) == false ){ return clean_exit( -1 ); }
       if ( find_boolean( &caspt2_calc,    line, "CASPT2_CALC"    ) == false ){ return clean_exit( -1 ); }
@@ -599,6 +603,16 @@ int main( int argc, char ** argv ){
          nvir = line.substr( pos, line.length() - pos );
       }
 
+      if ( line.find( "MOLCAS_ORDER" ) != string::npos ){
+         const int pos = line.find( "=" ) + 1;
+         molcas_order = line.substr( pos, line.length() - pos );
+      }
+
+      if ( line.find( "MOLCAS_REORDER" ) != string::npos ){
+         if ( am_i_master ){ cerr << "MOLCAS_REORDER is deprecated. Please use MOLCAS_ORDER and/or MOLCAS_FIEDLER." << endl; }
+         return clean_exit( -1 );
+      }
+
    }
    input.close();
 
@@ -712,7 +726,7 @@ int main( int argc, char ** argv ){
       if ( nvir_parsed[ cnt ] != 0 ){ full_active_space_calculation = false; }
    }
 
-   if ( ( molcas_2rdm.length() != 0 ) || ( molcas_3rdm.length() != 0 ) || ( molcas_f4rdm.length() != 0 ) || ( molcas_fock.length() != 0 ) ){
+   if ( ( molcas_2rdm.length() != 0 ) || ( molcas_3rdm.length() != 0 ) || ( molcas_f4rdm.length() != 0 ) || ( molcas_fock.length() != 0 ) || ( molcas_order.length() != 0 ) ){
       if ( full_active_space_calculation == false ){
          if ( am_i_master ){ cerr << "The options MOLCAS_* can only be specified for full active space calculations (when NOCC = NVIR = 0)!" << endl; }
          return clean_exit( -1 );
@@ -724,6 +738,21 @@ int main( int argc, char ** argv ){
       return clean_exit( -1 );
    }
 
+   /*********************************
+   *  Parse reordering if required  *
+   *********************************/
+
+   int * dmrg2ham = NULL;
+   if (( full_active_space_calculation == true ) && ( molcas_order.length() > 0 )){
+      const int list_length = count( molcas_order.begin(), molcas_order.end(), ',' ) + 1;
+      if ( list_length != fcidump_norb ){
+         if ( am_i_master ){ cerr << "The number of integers specified in MOLCAS_ORDER should equal the number of orbitals in the FCIDUMP file!" << endl; }
+         return clean_exit( -1 );
+      }
+      dmrg2ham = new int[ fcidump_norb ];
+      fetch_ints( molcas_order, dmrg2ham, fcidump_norb );
+   }
+
    /**********************
    *  Print the options  *
    ***********************/
@@ -749,7 +778,11 @@ int main( int argc, char ** argv ){
       cout << "   MOLCAS_3RDM        = " << molcas_3rdm << endl;
       cout << "   MOLCAS_F4RDM       = " << molcas_f4rdm << endl;
       cout << "   MOLCAS_FOCK        = " << molcas_fock << endl;
-      cout << "   MOLCAS_REORDER     = " << (( molcas_reorder ) ? "TRUE" : "FALSE" ) << endl;
+   if ( molcas_order.length() > 0 ){
+      cout << "   MOLCAS_ORDER       = [ " << dmrg2ham[ 0 ]; for ( int cnt = 1; cnt < fcidump_norb; cnt++ ){ cout << " ; " << dmrg2ham[ cnt ]; } cout << " ]" << endl;
+   } else {
+      cout << "   MOLCAS_FIEDLER     = " << (( molcas_fiedler ) ? "TRUE" : "FALSE" ) << endl;
+   }
       cout << "   MOLCAS_MPS         = " << (( molcas_mps ) ? "TRUE" : "FALSE" ) << endl;
    } else {
       cout << "   SCF_STATE_AVG      = " << (( scf_state_avg ) ? "TRUE" : "FALSE" ) << endl;
@@ -797,9 +830,9 @@ int main( int argc, char ** argv ){
       CheMPS2::Problem * prob = new CheMPS2::Problem( ham, multiplicity - 1, nelectrons, irrep );
 
       // Reorder the orbitals if desired
-      if (( group == 7 ) && ( molcas_reorder == false )){ prob->SetupReorderD2h(); }
-      if ( molcas_reorder ){
-         int * dmrg2ham = new int[ ham->getL() ];
+      if (( group == 7 ) && ( molcas_fiedler == false ) && ( molcas_order.length() == 0 )){ prob->SetupReorderD2h(); }
+      if (( molcas_fiedler ) && ( molcas_order.length() == 0 )){
+         dmrg2ham = new int[ ham->getL() ];
          if ( am_i_master ){
             const bool read_success = (( molcas_mps ) ? print_molcas_reorder( dmrg2ham, ham->getL(), "molcas_fiedler.txt", true ) : false );
             if ( read_success == false ){
@@ -815,6 +848,11 @@ int main( int argc, char ** argv ){
          prob->setup_reorder_custom( dmrg2ham );
          delete [] dmrg2ham;
       }
+      if ( molcas_order.length() > 0 ){
+         assert( fcidump_norb == ham->getL() );
+         prob->setup_reorder_custom( dmrg2ham );
+         delete [] dmrg2ham;
+      }
 
       CheMPS2::DMRG * dmrgsolver = new CheMPS2::DMRG( prob, opt_scheme, molcas_mps, tmp_folder );
 
