@@ -1,6 +1,6 @@
 /*
    CheMPS2: a spin-adapted implementation of DMRG for ab initio quantum chemistry
-   Copyright (C) 2013-2015 Sebastian Wouters
+   Copyright (C) 2013-2016 Sebastian Wouters
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -62,22 +62,12 @@ namespace CheMPS2{
          //! Getter for the number of variables in the vector " E_ij | FCI vector > " ; where irrep_center = I_i x I_j
          /** \param irrep_center The single electron excitation irrep I_i x I_j
              \return The number of variables in the corresponding vector */
-         unsigned long long getVecLength(const int irrep_center) const{ return irrep_center_jumps[ irrep_center ][ NumIrreps ]; }
-         
-         //! Get the number of irreps
-         /** \return The number of irreps */
-         unsigned int getNumIrreps() const{ return NumIrreps; }
+         unsigned int getVecLength(const int irrep_center) const{ return irrep_center_jumps[ irrep_center ][ num_irreps ]; }
          
          //! Get the target irrep
          /** \return The target irrep */
          int getTargetIrrep() const{ return TargetIrrep; }
          
-         //! Get the direct product of two irreps (Psi4's convention is used): product( irrep1 , irrep2 ) = irrep1 XOR irrep2.
-         /** \param Irrep1 The first irrep
-             \param Irrep2 The second irrep
-             \return The direct product Irrep1 x Irrep2 */
-         static int getIrrepProduct(const int Irrep1, const int Irrep2){ return Irrep1 ^ Irrep2; }
-         
          //! Get an orbital irrep
          /** \param orb The orbital index
              \return The irrep of orbital orb */
@@ -105,20 +95,44 @@ namespace CheMPS2{
          
          //! Calculates the FCI ground state with Davidson's algorithm
          /** \param inoutput If inoutput!=NULL, vector with getVecLength(0) variables which contains the initial guess at the start, and on exit the solution of the FCI calculation
-             \param DAVIDSON_NUM_VEC The maximum number of vectors to use in Davidson's algorithm; adjustable in case memory becomes an issue
+             \param DVDSN_NUM_VEC The maximum number of vectors to use in Davidson's algorithm; adjustable in case memory becomes an issue
              \return The ground state energy */
-         double GSDavidson(double * inoutput=NULL, const int DAVIDSON_NUM_VEC=CheMPS2::HEFF_DAVIDSON_NUM_VEC) const;
+         double GSDavidson(double * inoutput=NULL, const int DVDSN_NUM_VEC=CheMPS2::DAVIDSON_NUM_VEC) const;
          
          //! Return the global counter of the Slater determinant with the lowest energy
          /** \return The global counter of the Slater determinant with the lowest energy */
-         unsigned long long LowestEnergyDeterminant() const;
+         unsigned int LowestEnergyDeterminant() const;
          
-         //! Construct the (spin-summed) 2-RDM of a FCI vector: Gamma^2(i,j,k,l) = sum_sigma,tau < a^+_i,sigma a^+j,tau a_l,tau a_k,sigma > = TwoRDM[ i + L * ( j + L * ( k + L * l ) ) ]
+         //! Construct the (spin-summed) 2-RDM of a FCI vector: Gamma^2(i,j,k,l) = sum_sigma,tau < a^+_i,sigma a^+_j,tau a_l,tau a_k,sigma > = TwoRDM[ i + L * ( j + L * ( k + L * l ) ) ]
          /** \param vector The FCI vector of length getVecLength(0)
              \param TwoRDM To store the 2-RDM; needs to be of size getL()^4; point group symmetry shows in 2-RDM elements being zero
              \return The energy of the given FCI vector, calculated by contraction of the 2-RDM with Gmat and ERI */
          double Fill2RDM(double * vector, double * TwoRDM) const;
          
+         //! Construct the (spin-summed) 3-RDM of a FCI vector: Gamma^3(i,j,k,l,m,n) = sum_sigma,tau,s < a^+_{i,sigma} a^+_{j,tau} a^+_{k,s} a_{n,s} a_{m,tau} a_{l,sigma} > = ThreeRDM[ i + L * ( j + L * ( k + L * ( l + L * ( m + L * n ) ) ) ) ]
+         /** \param vector The FCI vector of length getVecLength(0)
+             \param ThreeRDM To store the 3-RDM; needs to be of size getL()^6; point group symmetry shows in 3-RDM elements being zero */
+         void Fill3RDM(double * vector, double * ThreeRDM) const;
+         
+         //! Construct the (spin-summed) 4-RDM of a FCI vector: Gamma^4(i,j,k,l,p,q,r,t) = sum_sigma,tau,s,z < a^+_{i,sigma} a^+_{j,tau} a^+_{k,s} a^+_{l,z} a_{t,z} a_{r,s} a_{q,tau} a_{p,sigma} > = FourRDM[ i + L * ( j + L * ( k + L * ( l + L * ( p + L * ( q + L * ( r + L * t ) ) ) ) ) ) ]
+         /** \param vector The FCI vector of length getVecLength(0)
+             \param FourRDM To store the 4-RDM; needs to be of size getL()^8; point group symmetry shows in 4-RDM elements being zero */
+         void Fill4RDM(double * vector, double * FourRDM) const;
+         
+         //! Construct the (spin-summed) contraction of the 4-RDM with the Fock operator: output(i,j,k,p,q,r) = sum_{l,t} Fock(l,t) * Gamma^4(i,j,k,l,p,q,r,t)
+         /** \param vector The FCI vector of length getVecLength(0)
+             \param ThreeRDM The spin-summed 3-RDM as calculated by Fill3RDM
+             \param Fock The symmetric Fock operator Fock(i,j) = Fock[ i + L * j ] = Fock[ j + L * i ]
+             \param output To store the contraction output(i,j,k,p,q,r) = output[ i + L * ( j + L * ( k + L * ( p + L * ( q + L * r ) ) ) ) ]; needs to be of size getL()^6; point group symmetry shows in elements being zero; has 12-fold permutation symmetry just like 3-RDM */
+         void Fock4RDM(double * vector, double * ThreeRDM, double * Fock, double * output) const;
+         
+         //! Construct part of the 4-RDM: output(i,j,k,p,q,r) = Gamma^4(i,j,k,z,p,q,r,z)
+         /** \param vector The FCI vector of length getVecLength(0)
+             \param three_rdm The spin-summed 3-RDM as calculated by Fill3RDM
+             \param orbz The orbital z which is fixed in Gamma^4(i,j,k,z,p,q,r,z)
+             \param output To store part of the 4-RDM output(i,j,k,p,q,r) = output[ i + L * ( j + L * ( k + L * ( p + L * ( q + L * r ) ) ) ) ]; needs to be of size getL()^6; point group symmetry shows in elements being zero; has 12-fold permutation symmetry just like 3-RDM */
+         void Diag4RDM( double * vector, double * three_rdm, const unsigned int orbz, double * output ) const;
+         
          //! Measure S(S+1) (spin squared)
          /** \param vector The FCI vector of length getVecLength(0)
              \return Measured value of S(S+1) */
@@ -127,12 +141,12 @@ namespace CheMPS2{
          //! Fill a vector with random numbers in the interval [-1,1[; used when output for GSDavidson is desired but no specific input can be given
          /** \param vecLength The length of the vector; when used for GSDavidson it should be getVecLength(0)
              \param vec The vector to fill with random numbers */
-         static void FillRandom(const unsigned long long vecLength, double * vec);
+         static void FillRandom(const unsigned int vecLength, double * vec);
          
          //! Set the entries of a vector to zero
          /** \param vecLength The vector length
              \param vec The vector which has to be set to zero */
-         static void ClearVector(const unsigned long long vecLength, double * vec);
+         static void ClearVector(const unsigned int vecLength, double * vec);
          
 //==========> Green's functions functionality
          
@@ -290,7 +304,7 @@ namespace CheMPS2{
          //! Function which performs the Hamiltonian times Vector product (without Econstant!!) making use of the (ij|kl) = (ji|kl) = (ij|lk) = (ji|lk) symmetry of the electron repulsion integrals
          /** \param input The vector of length getVecLength(0) on which the Hamiltonian should act
              \param output Vector of length getVecLength(0) which contains on exit the Hamiltonian times input */
-         void HamTimesVec(double * input, double * output) const;
+         void matvec( double * input, double * output ) const;
          
          //! Sandwich the Hamiltonian between two Slater determinants (return a specific element) (without Econstant!!)
          /** \param bits_bra_up Bit representation of the <bra| Slater determinant of the up (alpha) electrons (length L)
@@ -308,7 +322,7 @@ namespace CheMPS2{
              \param counter The given global counter corresponding to " E_ij | FCI vector > "
              \param bits_up Array of length L to store the bit representation of the up (alpha) electrons in
              \param bits_down Array of length L to store the bit representation of the down (beta) electrons in */
-         void getBitsOfCounter(const int irrep_center, const unsigned long long counter, int * bits_up, int * bits_down) const;
+         void getBitsOfCounter(const int irrep_center, const unsigned int counter, int * bits_up, int * bits_down) const;
          
          //! Convertor between two representations of a same spin-projection Slater determinant
          /** \param Lvalue The number of orbitals
@@ -326,41 +340,41 @@ namespace CheMPS2{
          /** \param irrep_center The single electron excitation irrep I_i x I_j
              \param counter The given global counter corresponding to " E_ij | FCI vector > "
              \return The corresponding irrep of the up Slater determinant */
-         int getUpIrrepOfCounter(const int irrep_center, const unsigned long long counter) const;
+         int getUpIrrepOfCounter(const int irrep_center, const unsigned int counter) const;
          
-//==========> Some lapack like routines which work with unsigned long long's (not necessary anymore as StartupIrrepCenter() checks "assert( max_integer >= maxVecLength );" )
+//==========> Some lapack like routines
 
          //! Take the inproduct of two vectors
          /** \param vecLength The vector length
              \param vec1 The first vector
              \param vec2 The second vector
              \return The inproduct < vec1 | vec2 > */
-         static double FCIddot(const unsigned long long vecLength, double * vec1, double * vec2);
+         static double FCIddot(const unsigned int vecLength, double * vec1, double * vec2);
          
          //! Copy a vector
          /** \param vecLength The vector length
              \param origin Vector to be copied
              \param target Where to copy the vector to */
-         static void FCIdcopy(const unsigned long long vecLength, double * origin, double * target);
+         static void FCIdcopy(const unsigned int vecLength, double * origin, double * target);
          
          //! Calculate the 2-norm of a vector
          /** \param vecLength The vector length
              \param vec The vector
              \return The 2-norm of vec */
-         static double FCIfrobeniusnorm(const unsigned long long vecLength, double * vec);
+         static double FCIfrobeniusnorm(const unsigned int vecLength, double * vec);
          
          //! Do lapack's daxpy vec_y += alpha * vec_x
          /** \param vecLength The vector length
              \param alpha The scalar factor
              \param vec_x The vector which has to be added to vec_y in rescaled form
              \param vec_y The target vector */
-         static void FCIdaxpy(const unsigned long long vecLength, const double alpha, double * vec_x, double * vec_y);
+         static void FCIdaxpy(const unsigned int vecLength, const double alpha, double * vec_x, double * vec_y);
          
          //! Do lapack's dscal vec *= alpha
          /** \param vecLength The vector length
              \param alpha The scalar factor
              \param vec The vector which has to be rescaled */
-         static void FCIdscal(const unsigned long long vecLength, const double alpha, double * vec);
+         static void FCIdscal(const unsigned int vecLength, const double alpha, double * vec);
          
 //==========> Protected functions regarding the Green's functions
          
@@ -399,24 +413,32 @@ namespace CheMPS2{
              \param out Array of size getVecLength(0), which contains on exit (alpha + beta * Hamiltonian) * in */
          void CGAlphaPlusBetaHAM(const double alpha, const double beta, double * in, double * out) const;
          
-         //! Calculate out = precon * [(alpha + beta * Hamiltonian)^2 + eta^2] * precon * in (Econstant is taken into account!!)
+         //! Calculate out = [(alpha + beta * Hamiltonian)^2 + eta^2] * in (Econstant is taken into account!!)
          /** \param alpha The parameter alpha of the operator
              \param beta The parameter beta of the operator
              \param eta The parameter eta of the operator
-             \param precon The diagonal preconditioner
              \param in On entry the vector of size getVecLength(0) on which the operator should be applied; unchanged on exit
-             \param out Array of size getVecLength(0), which contains on exit precon * [(alpha + beta * Hamiltonian)^2 + eta^2] * precon * in
-             \param temp Workspace of size getVecLength(0)
-             \param temp2 Workspace of size getVecLength(0) */
-         void CGOperator(const double alpha, const double beta, const double eta, double * precon, double * in, double * temp, double * temp2, double * out) const;
+             \param out Array of size getVecLength(0), which contains on exit [(alpha + beta * Hamiltonian)^2 + eta^2] * in
+             \param temp Workspace of size getVecLength(0) */
+         void CGoperator(const double alpha, const double beta, const double eta, double * in, double * temp, double * out) const;
          
-         //! Calculate (without approximation) precon = (diag[(alpha + beta * Hamiltonian)^2 + eta^2])^{ -1/2 } (Econstant is taken into account!!)
+         //! Calculate (without approximation) diagonal = diag[ (alpha + beta * Hamiltonian)^2 + eta^2 ] (Econstant is taken into account!!)
          /** \param alpha The parameter alpha of the operator
              \param beta The parameter beta of the operator
              \param eta The parameter eta of the operator
-             \param precon Array of size getVecLength(0), which contains on exit (diag[(alpha + beta * Hamiltonian)^2 + eta^2])^{ -1/2 }
+             \param diagonal Array of size getVecLength(0), which contains on exit diag[ (alpha + beta * Hamiltonian)^2 + eta^2 ]
              \param workspace Workspace of size getVecLength(0) */
-         void CGDiagPrecond(const double alpha, const double beta, const double eta, double * precon, double * workspace) const;
+         void CGdiagonal(const double alpha, const double beta, const double eta, double * diagonal, double * workspace) const;
+         
+//==========> Protected functions regarding the higher order RDMs
+
+         //! Apply E_{crea,anni} to |orig_vector> and store the result in |result_vector>
+         /** \param orig_vector The original vector
+             \param result_vector The result vector
+             \param crea The orbital index of the creator
+             \param anni The orbital index of the annihilator
+             \param orig_target_irrep The irrep of the orig_vector */
+         void apply_excitation( double * orig_vector, double * result_vector, const int crea, const int anni, const int orig_target_irrep ) const;
       
       private:
       
@@ -436,12 +458,12 @@ namespace CheMPS2{
          double * ERI;
          
          //! The number of irreps of the molecule's Abelian point group with real-valued character table
-         unsigned int NumIrreps;
+         unsigned int num_irreps;
          
-         //! The targeted irrep of the total FCI wavefunction ( 0 <= TargetIrrep < NumIrreps )
+         //! The targeted irrep of the total FCI wavefunction ( 0 <= TargetIrrep < num_irreps )
          int TargetIrrep;
          
-         //! Array of length L which contains for each orbital the irrep ( 0 <= orb < L : 0 <= orb2irrep[ orb ] < NumIrreps )
+         //! Array of length L which contains for each orbital the irrep ( 0 <= orb < L : 0 <= orb2irrep[ orb ] < num_irreps )
          int * orb2irrep;
          
          //! The number of orbitals ( Hubbard type orbitals with 4 possible occupations each )
@@ -472,25 +494,19 @@ namespace CheMPS2{
          unsigned int ** cnt2str_down;
          
          //! For irrep "irrep_result" and up (alpha) Slater determinant counter "result" lookup_cnt_alpha[ irrep_result ][ i + L * ( j + L * result ) ] returns the counter "origin" which corresponds to | result > = +/- E^{alpha}_ij | origin > or | origin > = +/- E^{alpha}_ji | result >
-         int ** lookup_cnt_alpha;
+         int *** lookup_cnt_alpha;
          
          //! For irrep "irrep_result" and down (beta) Slater determinant counter "result" lookup_cnt_beta[ irrep_result ][ i + L * ( j + L * result ) ] returns the counter "origin" which corresponds to | result > = +/- E^{beta}_ij | origin > or | origin > = +/- E^{beta}_ji | result >
-         int ** lookup_cnt_beta;
-         
-         //! For irrep "irrep_result" and up (alpha) Slater determinant counter "result" lookup_irrep_alpha[ irrep_result ][ i + L * ( j + L * result ) ] returns the irrep "irrep_origin" which corresponds to | result > = +/- E^{alpha}_ij | origin >
-         int ** lookup_irrep_alpha;
-         
-         //! For irrep "irrep_result" and down (beta) Slater determinant counter "result" lookup_irrep_beta[ irrep_result ][ i + L * ( j + L * result ) ] returns the irrep "irrep_origin" which corresponds to | result > = +/- E^{beta}_ij | origin >
-         int ** lookup_irrep_beta;
+         int *** lookup_cnt_beta;
          
          //! For irrep "irrep_result" and up (alpha) Slater determinant counter "result" lookup_sign_alpha[ irrep_result ][ i + L * ( j + L * result ) ] returns the sign s which corresponds to | result > = s * E^{alpha}_ij | origin >
-         int ** lookup_sign_alpha;
+         int *** lookup_sign_alpha;
          
          //! For irrep "irrep_result" and down (beta) Slater determinant counter "result" lookup_sign_beta[ irrep_result ][ i + L * ( j + L * result ) ] returns the sign s which corresponds to | result > = s * E^{beta}_ij | origin >
-         int ** lookup_sign_beta;
+         int *** lookup_sign_beta;
          
          //! For irrep_center = irrep_creator x irrep_annihilator the number of corresponding excitation pairs E_{creator <= annihilator} is given by irrep_center_num[ irrep_center ]
-         unsigned int *  irrep_center_num;
+         unsigned int * irrep_center_num;
          
          //! For irrep_center = irrep_creator x irrep_annihilator the creator orbital index of the nth (0 <= n < irrep_center_num[ irrep_center ]) excitation pair is given by irrep_center_crea_orb[ irrep_center ][ n ]
          unsigned int ** irrep_center_crea_orb;
@@ -499,7 +515,7 @@ namespace CheMPS2{
          unsigned int ** irrep_center_anni_orb;
          
          //! The global index corresponding to a vector " E_{ij} | FCI vector > " with irrep_center = irrep_i x irrep_j is given by irrep_center_jumps[ irrep_center ][ irrep_alpha ] + count_alpha + numPerIrrep_up[ irrep_alpha ] * cnt_beta where count_alpha and count_beta are the up (alpha) and down (beta) Slater determinants with resp. irreps irrep_alpha and irrep_beta = TargetIrrep x irrep_alpha x irrep_center
-         unsigned long long ** irrep_center_jumps;
+         unsigned int ** irrep_center_jumps;
          
          //! Number of doubles in each of the HVXworkbig arrays
          unsigned long long HXVsizeWorkspace;
@@ -521,6 +537,19 @@ namespace CheMPS2{
          
          //! Initialize a part of the private variables
          void StartupIrrepCenter();
+         
+         //! Actual routine used by Fill3RDM, Fock4RDM, Diag4RDM
+         double Driver3RDM(double * vector, double * output, double * three_rdm, double * fock, const unsigned int orbz) const;
+
+         //! Alpha excitation kernels
+         static void excite_alpha_omp( const unsigned int dim_new_up, const unsigned int dim_old_up, const unsigned int dim_down, double * origin, double * result, int * signmap, int * countmap );
+         static void excite_alpha_first( const unsigned int dim_new_up, const unsigned int dim_old_up, const unsigned int start_down, const unsigned int stop_down, double * origin, double * result, int * signmap, int * countmap );
+         static void excite_alpha_second_omp( const unsigned int dim_new_up, const unsigned int dim_old_up, const unsigned int start_down, const unsigned int stop_down, double * origin, double * result, int * signmap, int * countmap );
+
+         //! Beta excitation kernels
+         static void excite_beta_omp( const unsigned int dim_up, const unsigned int dim_new_down, double * origin, double * result, int * signmap, int * countmap );
+         static void excite_beta_first( const unsigned int dim_up, const unsigned int start_down, const unsigned int stop_down, double * origin, double * result, int * signmap, int * countmap );
+         static void excite_beta_second_omp( const unsigned int dim_up, const unsigned int start_down, const unsigned int stop_down, double * origin, double * result, int * signmap, int * countmap );
 
    };
 
