@@ -1,6 +1,6 @@
 /*
    CheMPS2: a spin-adapted implementation of DMRG for ab initio quantum chemistry
-   Copyright (C) 2013-2015 Sebastian Wouters
+   Copyright (C) 2013-2016 Sebastian Wouters
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -29,13 +29,7 @@ namespace CheMPS2{
    class Initialize{
 
       public:
-      
-         //! Constructor
-         Initialize();
-         
-         //! Destructor
-         virtual ~Initialize();
-         
+
          //! Initialize the random number generator and the cout.precision
          static void Init();
 
