@@ -28,7 +28,7 @@ Initializer.Init()
 
 # Read in the FCIDUMP
 psi4group = 5 # c2v: see chemps2/Irreps.h
-filename  = '../../tests/matrixelements/CH4.STO3G.FCIDUMP'
+filename  = b'../../tests/matrixelements/CH4.STO3G.FCIDUMP'
 orbirreps = np.array([-1, -1], dtype=ctypes.c_int) # CheMPS2 reads it in from FCIDUMP
 Ham = PyCheMPS2.PyHamiltonian( -1, psi4group, orbirreps, filename )
 
@@ -93,9 +93,9 @@ theFCI.Diag4RDM( GSvector, ThreeRDM, ham_orbz, fci_diag_4rdm )
 RMSerror4DM = np.linalg.norm( 0.5 * dmrg_diag_4rdm - fci_diag_4rdm )
 RMSerror2DM = np.sqrt(RMSerror2DM)
 RMSerror3DM = np.sqrt(RMSerror3DM)
-print "Frobenius norm of the difference of the DMRG and FCI 2-RDM =", RMSerror2DM
-print "Frobenius norm of the difference of the DMRG and FCI 3-RDM =", RMSerror3DM
-print "Frobenius norm of the difference of the DMRG and FCI diag(4-RDM) for fixed orbital", ham_orbz, "=", RMSerror4DM
+print("Frobenius norm of the difference of the DMRG and FCI 2-RDM =", RMSerror2DM)
+print("Frobenius norm of the difference of the DMRG and FCI 3-RDM =", RMSerror3DM)
+print("Frobenius norm of the difference of the DMRG and FCI diag(4-RDM) for fixed orbital", ham_orbz, "=", RMSerror4DM)
 del theFCI
 
 OptScheme.setInstruction(0, 1500, 1e-10,  3, 0.0)
@@ -114,7 +114,7 @@ del Initializer
 
 # Check whether the test succeeded
 if ((np.fabs(EnergyDMRG - EnergyFCI) < 1e-8) and (RMSerror2DM < 1e-3) and (RMSerror3DM < 1e-3) and (RMSerror4DM < 1e-3)):
-    print "================> Did test 10 succeed : yes"
+    print("================> Did test 10 succeed : yes")
 else:
-    print "================> Did test 10 succeed : no"
+    print("================> Did test 10 succeed : no")
 
